#!/usr/bin/perl -w
package Kpse;

use 5.006;
use strict;
use warnings;
use Carp;

require Exporter;
require DynaLoader;
use AutoLoader;
use Data::Dumper;
our @ISA = qw(Exporter DynaLoader);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Kpse ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	extend_filename
	find_suffix
	kpse_absolute_p
	kpse_all_path_search
	kpse_bitmap_tolerance
	kpse_brace_expand
	kpse_cnf_get
	kpse_db_insert
	kpse_db_search
	kpse_expand
	kpse_expand_default
	kpse_filename_component
	kpse_find_file
	kpse_find_glyph
	kpse_fontmap_lookup
	kpse_init_db
	kpse_init_fallback_resolutions
	kpse_init_format
	kpse_init_prog
	kpse_magstep_fix
	kpse_make_tex
	kpse_maketex_option
	kpse_open_file
	kpse_path_element
	kpse_path_expand
	kpse_path_search
	kpse_readable_file
	kpse_reset_program_name
	kpse_set_program_enabled
	kpse_set_program_name
	kpse_tex_hush
	kpse_tilde_expand
	kpse_truncate_filename
	kpse_var_expand
	kpse_var_value
	make_prefix
	make_suffix
	remove_suffix
	kpse_version
        kpse_total_formats
	kpse_format_info
	$kpathsea_debug
	$kpse_fallback_font
	$kpse_fallback_resolutions
	$kpse_fallback_resolutions_string
	$kpse_program_name
	$program_invocation_name
	$program_invocation_short_name
	@DEBUGSTR
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '0.01';

our($kpathsea_debug, $kpse_fallback_font, $kpse_fallback_resolutions, $kpse_fallback_resolutions_string, $kpse_program_name, $program_invocation_name, $program_invocation_short_name, @DEBUGSTR);

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    our $AUTOLOAD;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "& not defined" if $constname eq 'constant';
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/ || $!{EINVAL}) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
	    croak "Your vendor has not defined Kpse macro $constname";
	}
    }
    {
	no strict 'refs';
	# Fixed between 5.005_53 and 5.005_61
	if ($] >= 5.00561) {
	    *$AUTOLOAD = sub () { $val };
	}
	else {
	    *$AUTOLOAD = sub { $val };
	}
    }
    goto &$AUTOLOAD;
}

bootstrap Kpse $VERSION;
{
    _tievar_kpathsea_debug($kpathsea_debug);
    _tievar_kpse_fallback_font($kpse_fallback_font);
    _tievar_kpse_fallback_resolutions($kpse_fallback_resolutions);
    _tievar_kpse_fallback_resolutions_string($kpse_fallback_resolutions_string);
    _tievar_kpse_program_name($kpse_program_name);
    _tievar_program_invocation_name($program_invocation_name);
    _tievar_program_invocation_short_name($program_invocation_short_name);
}


# Preloaded methods go here.
our %types;
our %extensions;
our %paths;
our %names;
our %objects;
my $initdbdone = 0;
my @mktexlist = ();
my @nomktexlist = ();
our %mktexhash;
our %formats;

sub initialize {
  kpse_set_program_name("kpsewhich","tex"); # a default for init_format
  my $last = kpse_total_formats();
  $last--;
  foreach my $a (0..$last) {
    my $b = kpse_init_format($a);
    my $filep = kpse_format_info($a);
    $types{$filep->type()} = $a;
    $paths{$filep->type()} = $filep->path();
    $objects{$filep->type()} = $filep;
    $names{$a} = $filep->type();
    # init extensions hash
    @{$extensions{$filep->type()}} = (@{$filep->suffix()}, @{$filep->alt_suffix()});
    # init format lookup hash
    foreach my $ext (@{$extensions{$filep->type()}}){
      $formats{$ext} = $filep->type() unless defined $formats{$ext};
    }
  }
  # init mktex lists
  foreach my $key (qw(pk mf tex tfm ocp ofm)) {
    $mktexhash{$key} = $objects{$key}->program_enabled_p() || 0;
  }
  @nomktexlist = grep { $mktexhash{$_} == 0 } keys %mktexhash;
  @mktexlist =  grep { $mktexhash{$_} == 1 } keys %mktexhash;
}


sub name_index {
  my $a = shift;
  return $names{$a} if defined $names{$a};
  return undef;
}

sub type_index {
  my $a = shift;
  return $types{$a} if defined $types{$a};
  return undef;
}

sub debug {
  my $a =  $kpathsea_debug;
  my $b = shift;
  $kpathsea_debug = $b if defined $b;
  return $a;
}

our $dpi = 600;

sub dpi {
  my $a = $dpi;
  my $b = shift;
  $dpi = $b if defined $b;
  return $a;
}

our $path = '';

sub path {
  my $a = $path;
  my $b = shift;
  $path = $b if defined $b;
  return $a;
}

sub expand_braces {
  return kpse_brace_expand(shift);
}

sub expand_path {
  return kpse_path_expand(shift);
}

sub expand_var {
  return kpse_var_expand(shift);
}

our $mode = '';

sub mode {
  my $a = $mode;
  my $b = shift;
  $mode = $b if defined $b;
  return $a;
}

sub progname {
  my ($a,$b) = @_;
  $b = $a unless $b;
  kpse_reset_program_name($b);
  kpse_set_program_name($a,$b);
  kpse_init_prog(uc($b),$dpi,$mode,0);
}

sub show_path {
  my $a = shift;
  return $paths{$a};
}

sub version {
  return kpse_version();
}

sub debug_info {
  my @a = @DEBUGSTR;
  @DEBUGSTR = ();
  return @a;
}

sub types {
  return sort { $types{$a} <=> $types{$b}} keys %types;
}

sub extensions {
  my $a = shift;
  return $extensions{$a};
}

sub fontname_lookup {
  return kpse_fontname_lookup(shift);
}


sub init_db {
  kpse_init_db();
  $initdbdone=1;
}

sub open_file {
  my ($type,$filename) = @_;
  init_db() unless $initdbdone;
  return kpse_open_file($filename,$types{$type});
}

sub nomktex {
  my (@todos) = @_;
  return @nomktexlist unless @todos;
  foreach my $key (@todos) {
    if ($key =~ /^(pk|mf|tex|tfm|ocp|ofm)$/) {
      kpse_set_program_enabled($types{$key},0,4); # last arg is a small number indicating type
      $mktexhash{$key} = 0;
    }
  }
  foreach my $key (qw(pk mf tex tfm ocp ofm)) {
    $mktexhash{$key} = $objects{$key}->program_enabled_p() || 0;
  }
  @nomktexlist = grep { $mktexhash{$_} == 0 } keys %mktexhash;
  @mktexlist =  grep { $mktexhash{$_} == 1 } keys %mktexhash;
  return @nomktexlist;
}

sub mktex {
  my (@todos) = @_;
  return @mktexlist unless @todos;
  foreach my $key (@todos) {
    if ($key =~ /^(pk|mf|tex|tfm|ocp|ofm)$/) {
      kpse_set_program_enabled($types{$key},1,4); # last arg is a small number indicating type
      $mktexhash{$key} = 1;
    }
  }
  foreach my $key (qw(pk mf tex tfm ocp ofm)) {
    $mktexhash{$key} = $objects{$key}->program_enabled_p() || 0;
  }
  @nomktexlist = grep { $mktexhash{$_} == 0 } keys %mktexhash;
  @mktexlist =  grep { $mktexhash{$_} == 1 } keys %mktexhash;
  return @mktexlist;
}

sub lookup {
  my ($format,$filename,$mustexist) = @_;  
  if ($path) {
    if (wantarray()) {
      return kpse_all_path_search($path,$filename);
    } else {
      return kpse_path_search($path,$filename,$mustexist);
    }
  } else {
    my $ext;
    my $tdpi;
    unless ($format) { 
      if ($filename eq 'ls-R') { $ext = 'ls-R' }
      elsif ($filename eq 'psfonts.map') { $ext = '.pro' }
      else { $filename =~ /\.([0-9]*)([a-zA-Z]+?)$/;
	     $ext = $2; 
	     $tdpi = $1;
	     $ext = ".$ext" unless $tdpi; }
      $format = $formats{$ext} || 'tex';
    };
    if ($format eq 'gf' || $format eq 'pk' || $format eq 'bitmap font') {
      my @ret = find_glyph_file($format,$filename,$tdpi || $dpi);
      return $ret[0];
    } else {
      return find_file($format,$filename,$mustexist);
    }
  }
}


sub find_file {
  my ($type,$filename,$mustbe) = @_;
  $mustbe = 0 unless defined $mustbe;
  init_db() unless $initdbdone;
  if (wantarray()) {
    my @temp;
    my @results;
    my $b = kpse_init_format($types{$type});
    my @dirs = split /:/,$b;
    foreach my $dir (@dirs) {
      my $dirx = $dir;
      $dirx =~ s/^!!//;
      @temp = @{kpse_db_search ($filename, $dirx, 1)};
      if(@temp) {
	@results = (@results, @temp);
      } else {
	@results = (@results, @{kpse_all_path_search($dir,$filename)}) if $dir eq $dirx;
      }
    }
    return @results;
  } else {
    my $result;
    $result = kpse_find_file($filename,$types{$type},$mustbe);
    return $result;
  }
}

sub find_glyph_file {
  my ($type,$filename,$tdpi,$path) = @_;
  $tdpi = $dpi unless defined $tdpi;
  init_db() unless $initdbdone;
  my $oldpath=0;
  my $buffer = kpse_glyph_file_type->new();
  my $glyphfile = $buffer->_to_ptr();
  my $name = kpse_find_glyph($filename,$tdpi,$types{$type},$glyphfile);
  my $filep = Kpse::kpse_format_info($glyphfile->format());
  my $ntype = $filep->type();
  my @ret = ($name, $glyphfile->name(),
	     $glyphfile->source(),
             $ntype,
	     $glyphfile->dpi());
  return @ret;
}


# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Kpse - Perl extension for the Kpathsea search library

=head1 SYNOPSIS

  use Kpse;
  my $succes     = Kpse::initialize();
  my $oldvalue   = Kpse::debug(NUM);
  my $olddpi     = Kpse::dpi(NUM);
  my $expanded   = Kpse::expand_braces(STRING);
  my $expanded   = Kpse::expand_path(STRING);
  my $expanded   = Kpse::expand_var(STRING);
  my $succes     = Kpse::mode(STRING);
  my @oldlist    = Kpse::mktex(FMTLIST);
  my @oldlist    = Kpse::nomktex(FMTLIST);
  my $oldpath   = Kpse::path(PATH);
  my $succes   = Kpse::progname(STRING,[STRING]);
  my $path   = Kpse::show_path(NAME);
  my $version   = Kpse::version();
  my @fullname = Kpse::lookup(FORMATNAME,FILENAME,[must_exist]);

  my $fullname = Kpse::find_file(NAME,FILENAME,[must_exist]);
  my @fullnames   = Kpse::find_file(NAME,FILENAME,[must_exist]);
  my $HANDLE   = Kpse::open_file(NAME,FILENAME);
  my ($fullname,$name,$format,$source,$dpi)   = Kpse::find_glyph_file(NAME,FILENAME,[dpi,path]);
  my @strings    = Kpse::debug_info();
  my @NAMES    = Kpse::types();
  my @extensions = Kpse::extensions(NAME);
  my $fontname = Kpse::fontmap_lookup(PSFONTNAME)

=head1 ABSTRACT

This package provides a perl interface to the Kpathsea library.
Most of the user-level functions are designed to mimic kpsewhich
behaviour, but low-level libarary interface functions are also available.

=head1 DESCRIPTION


=head2 EXPORT

None by default.

-head2 User interface functions

 boolean initialize (string progname)

=head2 Internal functions

  string extend_filename (const_string name, const_string suffix)
  string find_suffix (const_string name)
  boolean kpse_absolute_p (const_string filename, boolean relative_ok)
  arrayref kpse_all_path_search (const_string path, const_string name)
  arrayref kpse_db_search (const_string name, const_string path_elt, boolean all)
  boolean kpse_bitmap_tolerance (double dpi1, double dpi2)
  string kpse_brace_expand (const_string path)
  string kpse_cnf_get (const_string var)
  void kpse_db_insert (const_string fname)
  string kpse_expand (const_string s)
  string kpse_expand_default (const_string path, const_string dflt)
  string kpse_filename_component (const_string path)
  string kpse_find_file (const_string name, kpse_file_format_type format, boolean must_exist)
  string kpse_find_glyph (const_string font_name, unsigned dpi, kpse_file_format_type format, kpse_glyph_file_type *glyph_file)
  hashref kpse_fontmap_lookup (const_string key)
  void kpse_init_db (void)
  void kpse_init_fallback_resolutions (string envvar)
  const_string kpse_init_format (kpse_file_format_type)
  void kpse_init_prog (const_string prefix, unsigned dpi, const_string mode, const_string fallback)
  unsigned kpse_magstep_fix (unsigned dpi, unsigned bdpi, int *m_ret)
  string kpse_make_tex (kpse_file_format_type format, const_string base_file)
  void kpse_maketex_option (const_string fmtname, boolean value)
  HANDLE kpse_open_file (const_string name, kpse_file_format_type)
  string kpse_path_element (const_string path)
  string kpse_path_expand (const_string path)
  string kpse_path_search (const_string path, const_string name, boolean must_exist)
  string kpse_readable_file (const_string name)
  void kpse_reset_program_name (const_string progname)
  void kpse_set_program_enabled (kpse_file_format_type fmt, boolean value, kpse_src_type level)
  void kpse_set_program_name (const_string argv0, const_string progname)
  boolean kpse_tex_hush (const_string what)
  string kpse_tilde_expand (const_string filename)
  string kpse_truncate_filename (const_string name)
  string kpse_var_expand (const_string src)
  string kpse_var_value (const_string var)
  string make_prefix (string stem_prefix, string name)
  string make_suffix (const_string s, const_string suffix)
  string remove_suffix (const_string name)
  string kpse_version (void)
  unsigned kpse_total_formats (void)
  kpse_format_infoPtr kpse_format_info(unsigned index)

=head2 Exportable variables

  $kpathsea_debug
  $kpse_fallback_font
  $kpse_fallback_resolutions
  $kpse_fallback_resolutions_string
  $kpse_program_name
  $program_invocation_name
  $program_invocation_short_name
  @DEBUGSTR

=head2 Object and class methods for C<kpse_format_info_type>/C<kpse_format_info_typePtr>

The principal Perl representation of a C object of type C<kpse_format_info_type> is an
object of class C<kpse_format_info_typePtr> which is a reference to an integer
representation of a C pointer.  To create such an object, one may use
a combination

  my $buffer = kpse_format_info_type->new();
  my $obj = $buffer->_to_ptr();

This exersizes the following two methods, and an additional class
C<kpse_format_info_type>, the internal representation of which is a reference to a
packed string with the C structure.  Keep in mind that $buffer should
better survive longer than $obj.

=over

=item C<$object_of_type_kpse_format_info_type-E<gt>_to_ptr()>

Converts an object of type C<kpse_format_info_type> to an object of type C<kpse_format_info_typePtr>.

=item C<kpse_format_info_type-E<gt>new()>

Creates an empty object of type C<kpse_format_info_type>.  The corresponding packed
string is zeroed out.

=item C<type(...)>, C<path(...)>, C<raw_path(...)>, C<path_source(...)>, C<override_path(...)>, C<client_path(...)>, C<cnf_path(...)>, C<default_path(...)>, C<suffix(...)>, C<alt_suffix(...)>, C<suffix_search_only(...)>, C<program(...)>, C<program_args(...)>, C<program_enabled_p(...)>, C<program_enable_level(...)>, C<binmode(...)>

return the current value of the corresponding element if called
without additional arguments.  Set the element to the supplied value
(and return the new value) if called with an additional argument.

Applicable to objects of type C<kpse_format_info_typePtr>.

=back


=head2 Object and class methods for C<kpse_glyph_file_type>/C<kpse_glyph_file_typePtr>

The principal Perl representation of a C object of type C<kpse_glyph_file_type> is an
object of class C<kpse_glyph_file_typePtr> which is a reference to an integer
representation of a C pointer.  To create such an object, one may use
a combination

  my $buffer = kpse_glyph_file_type->new();
  my $obj = $buffer->_to_ptr();

This exersizes the following two methods, and an additional class
C<kpse_glyph_file_type>, the internal representation of which is a reference to a
packed string with the C structure.  Keep in mind that $buffer should
better survive longer than $obj.

=over

=item C<$object_of_type_kpse_glyph_file_type-E<gt>_to_ptr()>

Converts an object of type C<kpse_glyph_file_type> to an object of type C<kpse_glyph_file_typePtr>.

=item C<kpse_glyph_file_type-E<gt>new()>

Creates an empty object of type C<kpse_glyph_file_type>.  The corresponding packed
string is zeroed out.

=item C<name(...)>, C<dpi(...)>, C<format(...)>, C<source(...)>

return the current value of the corresponding element if called
without additional arguments.  Set the element to the supplied value
(and return the new value) if called with an additional argument.

Applicable to objects of type C<kpse_glyph_file_typePtr>.

=back

=head1 AUTHOR

Taco Hoekwater, E<lt>taco@elvenkind.comE<gt>

=head1 SEE ALSO

L<perl>, L<kpathsea>.

=cut
